@@ -1229,31 +1229,6 @@ get_object_list (ECalBackendSync *backend, EDataCal *cal,
 	return GNOME_Evolution_Calendar_Success;
 }
 
-ECalBackendSyncStatus
-get_timezone (ECalBackendSync *backend, EDataCal *cal,
-	      const gchar *tzid, gchar **object)
-{
-	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
-	icaltimezone *zone;
-	icalcomponent *vtzcomp;
-	gchar *ical_obj;
-
-	d(printf("ecbe_get_timezone(%p, %p, %s)\n", backend, cal, tzid));
-
-	zone = g_hash_table_lookup (cbex->priv->timezones, tzid);
-	if (!zone)
-		return GNOME_Evolution_Calendar_ObjectNotFound;
-
-	vtzcomp = icaltimezone_get_component (zone);
-	if (!vtzcomp)
-		return GNOME_Evolution_Calendar_OtherError;
-
-	ical_obj = icalcomponent_as_ical_string_r (vtzcomp);
-	*object = ical_obj;
-
-	return  GNOME_Evolution_Calendar_Success;
-}
-
 icaltimezone *
 e_cal_backend_exchange_get_default_time_zone (ECalBackendSync *backend)
 {
@@ -1752,18 +1727,17 @@ get_attachment (ECalBackendExchange *cbex, const gchar *uid,
 			part = camel_multipart_get_part (multipart, i);
 			filename = camel_mime_part_get_filename (part);
 			if (filename) {
-				CamelStreamMem *stream_mem;
+				GByteArray *mem_bytes = g_byte_array_new ();
 
 				content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
 
-				stream = camel_stream_mem_new ();
-				stream_mem = (CamelStreamMem *)stream;
+				stream = camel_stream_mem_new_with_byte_array (mem_bytes);
 
 				camel_data_wrapper_decode_to_stream (content, stream);
-				attach_data = g_memdup (stream_mem->buffer->data, stream_mem->buffer->len);
+				attach_data = g_memdup (mem_bytes->data, mem_bytes->len);
 				attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 				// Attach
-				attach_file_url = save_attach_file (attach_file, (gchar *) attach_data, stream_mem->buffer->len);
+				attach_file_url = save_attach_file (attach_file, (gchar *) attach_data, mem_bytes->len);
 				g_free (attach_data);
 				g_free (attach_file);
 				d(printf ("attach file name : %s\n", attach_file_url));
@@ -1960,13 +1934,14 @@ gchar *
 build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject, gchar **boundary)
 {
 	CamelMimeMessage *msg;
-	CamelStreamMem *content;
+	CamelStream *mem_content;
 	CamelMimePart *mime_part;
 	CamelDataWrapper *dw, *wrapper;
 	CamelMultipart *multipart;
 	CamelInternetAddress *from;
 	CamelStream *stream;
 	CamelContentType *type;
+	GByteArray *mem_bytes;
 	const gchar *uid;
 	gchar *buffer = NULL, *cid;
 	gchar *from_name = NULL, *from_email = NULL;
@@ -2075,13 +2050,13 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 	camel_medium_set_content_object (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (multipart));
 	camel_object_unref (multipart);
 
-	content = (CamelStreamMem *)camel_stream_mem_new();
+	mem_bytes = g_byte_array_new ();
+	mem_content = camel_stream_mem_new_with_byte_array (mem_bytes);
 	dw = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
-	camel_data_wrapper_decode_to_stream(dw, (CamelStream *)content);
-	buffer = g_memdup (content->buffer->data, content->buffer->len);
-	buffer[content->buffer->len] = '\0';
+	camel_data_wrapper_decode_to_stream (dw, mem_content);
+	buffer = g_strndup ((const gchar *)mem_bytes->data, mem_bytes->len);
 	d(printf ("|| Buffer: \n%s\n", buffer));
-	camel_object_unref (content);
+	camel_object_unref (mem_content);
 	camel_object_unref (msg);
 
 	return buffer;
@@ -2159,11 +2134,19 @@ static icaltimezone *
 internal_get_timezone (ECalBackend *backend, const gchar *tzid)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	icaltimezone *zone;
 
-	if (!tzid)
-		return NULL;
+	g_return_val_if_fail (cbex != NULL, NULL);
+	g_return_val_if_fail (tzid != NULL, NULL);
 
-	return g_hash_table_lookup (cbex->priv->timezones, tzid);
+	/* search in backend's time zone cache */
+	zone = g_hash_table_lookup (cbex->priv->timezones, tzid);
+
+	/* if not found, chain up */
+	if (!zone && E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone)
+		zone = E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone (backend, tzid);
+
+	return zone;
 }
 
 icaltimezone *
@@ -2171,8 +2154,14 @@ e_cal_backend_exchange_lookup_timezone (const gchar *tzid,
 					gconstpointer custom,
 					GError **error)
 {
-	return internal_get_timezone (E_CAL_BACKEND ((ECalBackendExchange *)custom),
-				      tzid);
+	icaltimezone *zone = internal_get_timezone (E_CAL_BACKEND ((ECalBackendExchange *)custom), tzid);
+
+	/* The UTC timezone is a fallback, which is not supposed
+	   to be returned in call of e_cal_check_timezones, thus skip it */
+	if (zone && zone == icaltimezone_get_utc_timezone ())
+		zone = NULL;
+
+	return zone;
 }
 
 static void
@@ -2284,7 +2273,6 @@ class_init (ECalBackendExchangeClass *klass)
 	sync_class->get_default_object_sync = get_default_object;
 	sync_class->get_object_sync = get_object;
 	sync_class->get_object_list_sync = get_object_list;
-	sync_class->get_timezone_sync = get_timezone;
 	sync_class->add_timezone_sync = add_timezone;
 	sync_class->set_default_timezone_sync = set_default_timezone;
 	sync_class->get_freebusy_sync = get_freebusy;
