@@ -26,7 +26,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "camel-exchange-transport.h"
-#include "camel-stub.h"
+#include "camel-exchange-utils.h"
 
 #include <camel/camel-data-wrapper.h>
 #include <camel/camel-exception.h>
@@ -38,15 +38,12 @@
 
 #include <string.h>
 
-extern CamelStub *das_global_camel_stub;
-
 static gboolean exchange_send_to (CamelTransport *transport,
 				  CamelMimeMessage *message,
 				  CamelAddress *from,
 				  CamelAddress *recipients,
 				  CamelException *ex);
 
-
 static void
 camel_exchange_transport_class_init (CamelExchangeTransportClass *camel_exchange_transport_class)
 {
@@ -82,7 +79,6 @@ camel_exchange_transport_get_type (void)
 	return camel_exchange_transport_type;
 }
 
-
 static gboolean
 exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 		  CamelAddress *from, CamelAddress *recipients,
@@ -90,17 +86,18 @@ exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 {
 	CamelService *service = CAMEL_SERVICE (transport);
 	CamelStore *store = NULL;
-	char *url_string;
+	gchar *url_string;
 	const CamelInternetAddress *cia;
-	const char *addr;
+	const gchar *addr;
 	GPtrArray *recipients_array;
 	gboolean success;
 	CamelStream *stream;
 	CamelStreamFilter *filtered_stream;
 	CamelMimeFilter *crlffilter;
 	struct _camel_header_raw *header;
+	GByteArray *buffer;
 	GSList *h, *bcc = NULL;
-	int len, i;
+	gint len, i;
 
 	url_string = camel_session_get_password (service->session, service, NULL,
 						"ignored", "popb4smtp_uri", 0, ex);
@@ -113,6 +110,8 @@ exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 		return FALSE;
 	}
 
+	g_free (url_string);
+
 	recipients_array = g_ptr_array_new ();
 	len = camel_address_length (recipients);
 	cia = CAMEL_INTERNET_ADDRESS (recipients);
@@ -121,32 +120,20 @@ exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Cannot send message: one or more invalid recipients"));
 			g_ptr_array_free (recipients_array, TRUE);
-			g_free (url_string);
 			return FALSE;
 		}
-		g_ptr_array_add (recipients_array, (char *)addr);
+		g_ptr_array_add (recipients_array, (gchar *)addr);
 	}
 
 	if (!camel_internet_address_get (CAMEL_INTERNET_ADDRESS (from), 0, NULL, &addr)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				     _("Could not find 'From' address in message"));
 		g_ptr_array_free (recipients_array, TRUE);
-		g_free (url_string);
 		return FALSE;
 	}
 
-	if (!das_global_camel_stub) {
-		store = camel_session_get_store (service->session, url_string, ex);
-		if (!store) {
-			g_ptr_array_free (recipients_array, TRUE);
-			g_free (url_string);
-			return FALSE;
-		}
-		g_return_val_if_fail (das_global_camel_stub, FALSE);
-	}
-	g_free (url_string);
-
-	stream = camel_stream_mem_new ();
+	buffer = g_byte_array_new ();
+	stream = camel_stream_mem_new_with_byte_array (buffer);
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_ENCODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_ONLY);
 	filtered_stream = camel_stream_filter_new_with_stream (stream);
 	camel_stream_filter_add (filtered_stream, CAMEL_MIME_FILTER (crlffilter));
@@ -160,9 +147,9 @@ exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 			bcc = g_slist_append (bcc, g_strdup (header->value));
 		header = header->next;
 	}
-	
+
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "Bcc");
-	
+
 	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message),
 					    CAMEL_STREAM (filtered_stream));
 	camel_stream_flush (CAMEL_STREAM (filtered_stream));
@@ -179,12 +166,7 @@ exchange_send_to (CamelTransport *transport, CamelMimeMessage *message,
 		g_slist_free (bcc);
 	}
 
-	success = camel_stub_send (das_global_camel_stub, ex,
-				   CAMEL_STUB_CMD_SEND_MESSAGE,
-				   CAMEL_STUB_ARG_STRING, addr,
-				   CAMEL_STUB_ARG_STRINGARRAY, recipients_array,
-				   CAMEL_STUB_ARG_BYTEARRAY, CAMEL_STREAM_MEM (stream)->buffer,
-				   CAMEL_STUB_ARG_END);
+	success = camel_exchange_utils_send_message (CAMEL_SERVICE (transport), addr, recipients_array, buffer, ex);
 
 	g_ptr_array_free (recipients_array, TRUE);
 	camel_object_unref (CAMEL_OBJECT (stream));
